dnl
dnl  This file is part of Siena, a wide-area event notification system.
dnl  See http://www.inf.usi.ch/carzaniga/siena/
dnl
dnl  Author: Antonio Carzaniga
dnl  See the file AUTHORS for full details. 
dnl
dnl  Copyright (C) 2001-2003 University of Colorado
dnl
dnl  This program is free software; you can redistribute it and/or
dnl  modify it under the terms of the GNU General Public License
dnl  as published by the Free Software Foundation; either version 2
dnl  of the License, or (at your option) any later version.
dnl
dnl  This program is distributed in the hope that it will be useful,
dnl  but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl  GNU General Public License for more details.
dnl
dnl  You should have received a copy of the GNU General Public License
dnl  along with this program; if not, write to the Free Software
dnl  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
dnl  USA, or send email to one of the authors.
dnl
dnl $Id: acinclude.m4,v 1.14 2010-03-11 15:16:37 carzanig Exp $
dnl
dnl
dnl AC_OPT_PROFILING
dnl
AC_DEFUN([AC_OPT_PROFILING], [
AC_ARG_ENABLE(profiling, 
  AC_HELP_STRING([--enable-profiling],
	[include profiling information. Values are "yes", "coverage" and "no" (default is "no")]),
dnl this is to optionally compile with profiling
dnl I don't know too much about this, but it looks like
dnl -pg only works with static libraries, so I'm going to 
dnl disable shared libraries here.
  [ case "$enableval" in
        coverage )
	    CFLAGS_prof='-pg -fprofile-arcs -ftest-coverage'
	    CXXFLAGS_prof='-pg -fprofile-arcs -ftest-coverage'
	    LDFLAGS_prof='-pg'
	    LIBS_prof='-lgcov'
	    AC_MSG_RESULT([Enabling profiling and coverage information])
	    ;;
        * )
	    CFLAGS_prof='-pg'
	    CXXFLAGS_prof='-pg'
	    LDFLAGS_prof='-pg'
	    LIBS_prof=''
	    AC_MSG_RESULT([Enabling profiling information])
	    ;;
    esac
    AC_DISABLE_SHARED ], 
  [ CFLAGS_prof=''
    CXXFLAGS_prof=''
    LDFLAGS_prof=''
    LIBS_prof=''
    AC_ENABLE_SHARED ])
AC_SUBST(CFLAGS_prof)
AC_SUBST(CXXFLAGS_prof)
AC_SUBST(LDFLAGS_prof)
AC_SUBST(LIBS_prof)
AC_DEFINE(WITH_PROFILING, 1, [Using gprof, so do not mess with SIGPROF])
])
dnl
dnl AC_OPT_ASSERTIONS
dnl
AC_DEFUN([AC_OPT_ASSERTIONS], [
AC_ARG_ENABLE(assertions, 
  AC_HELP_STRING([--enable-assertions],
	[enable evaluation of assertions (default is NO)]), ,
  [ AC_DEFINE(NDEBUG, 1, [Disables assertions and other debugging code])])
])
dnl
dnl AC_OPT_DEBUGGING
dnl
AC_DEFUN([AC_OPT_DEBUGGING], [
AC_ARG_ENABLE(debugging, 
  AC_HELP_STRING([--enable-debugging],
	[enable debug messages of assertions (default is NO)]), 
  [ AC_DEFINE(DEBUG_OUTPUT, 1, [Enables debugging output])])
])
dnl
dnl AC_OPT_STD_BITSET
dnl
AC_DEFUN([AC_OPT_STD_BITSET], [
ac_std_bitset=no
AC_ARG_ENABLE(std_bitset, 
  AC_HELP_STRING([--enable-std-bitset],
	[enable the use of the C++ standard <bitset> (default is NO)]),
  [ ac_std_bitset=yes ])
if test "$ac_std_bitset" = yes; then
  AC_DEFINE(USE_STD_BITSET, 1, [Enables the use of C++ standard bitset])
fi
])
dnl
dnl AC_CHECK_RDTSC([action-if-available [, action-if-not-available])
dnl
AC_DEFUN([AC_CHECK_RDTSC], [
AC_CACHE_CHECK([for the rdtsc instruction], [ac_cv_rdtsc], [
AC_COMPILE_IFELSE(AC_LANG_SOURCE([[
long long int rdtsc() {
    long long int x;
    __asm__ volatile ("rdtsc" : "=A" (x));
    return x;
}
]]), [ ac_cv_rdtsc=yes ], [ ac_cv_rdtsc=no ])])
case "$ac_cv_rdtsc" in
    yes)
	ifelse([$1], , :, [$1])
	;;
    *)
	ifelse([$2], , :, [$2])
	;;
esac
])
dnl
dnl AC_CHECK_RDTSC([action-if-available [, action-if-not-available])
dnl
AC_DEFUN([AC_CHECK_MFTB], [
AC_CACHE_CHECK([for the mftb instruction], [ac_cv_mftb], [
AC_COMPILE_IFELSE(AC_LANG_SOURCE([[
unsigned long get_cycles(void) {
    unsigned long ret;

    __asm__ __volatile__("mftb %0" : "=r" (ret) : );
    return ret;
}
]]), [ ac_cv_mftb=yes ], [ ac_cv_mftb=no ])])
case "$ac_cv_mftb" in
    yes)
	ifelse([$1], , :, [$1])
	;;
    *)
	ifelse([$2], , :, [$2])
	;;
esac
])
