// -*- C++ -*-
//
//  This file is part of Siena, a wide-area event notification system.
//  See http://www.inf.usi.ch/carzaniga/siena/
//
//  Authors: Antonio Carzaniga
//  See the file AUTHORS for full details. 
//
//  Copyright (C) 2002 University of Colorado
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
//  USA, or send email to one of the authors.
//
//
// $Id: bv.cc,v 1.4 2010-03-11 15:16:46 carzanig Exp $
//
#include <iostream>

#include <siena/allocator.h>

#include "bitvector.h"

using namespace std;
using namespace siena;

FTAllocator Mem;

istream & operator >> (istream & is, ibitvector & v) {
    size_t pos = 0;
    char c;
    while (is.get(c)) 
	if (c == '1' || c == '0') break;

    do {
	switch (c) {
	case '1': v.set(pos, Mem);
	case '0': ++pos; break;
	default: return is;
	}
    } while(is.get(c));

    return is;
}

ostream & operator <<(ostream & os, const bitvector & v) {
    for(size_t pos = 0; pos < v.get_size(); ++pos)
	os << (v.test(pos) ? '1' : '0');
    return os;
}

ostream & operator <<(ostream & os, const ibitvector & v) {
    for(size_t pos = 0; pos < v.get_size(); ++pos)
	os << (v.test(pos) ? '1' : '0');
    return os;
}

void compute_union(const ibitvector & x, const ibitvector & y) {
    bitvector res(x.get_size() > y.get_size() ? x.get_size() : y.get_size());
    res.set(x);
    cout << "Xs=" << x.get_size() << " Xc=" << x.get_count() << " X=" << x<< endl;
    cout << "Ys=" << y.get_size() << " Yc=" << y.get_count() << " Y=" << y<< endl;
    cout << "RXs=" << res.get_size() << " RXc=" << res.get_count() << " RX=" << res << endl;
    res.set(y);
    cout << "RXYs=" << res.get_size() << " RXYc=" << res.get_count() << " RXY=" << res << endl;
}

int main(int argc, char *argv[]) {
    if (argc < 2) {
	ibitvector x;
	ibitvector y;
	while(cin >> x >> y) {
	    compute_union(x,y);
	    x.clear();
	    y.clear();
	}
	return 0;
    } else {
	ibitvector x;
	int i;
	int max = 0;
	for(;;) {
	    while(cin >> i && i >= 0) {
		if (i > max) max = i;
		x.set(i, Mem);
	    }
	    if (!cin) return 0;
	    cout << "->";
	    for(i=0; i <= max; ++i) 
		if (x.test(i)) cout << ' ' << i;
	    cout << endl;
	    x.clear();
	}
    }
}
